#include "global.h"
#include "mal.h"
#include "main.h"
#include "c_main.h"
#include "i_main.h"
#include "w_main.h"
#include "k_stdtype.h"
#include "init.h"
#include "version.h"
#include "w_app.h"
#include "w_tmr.h"
#include "io.h"
#include "tmr.h"
#include "w_io.h"
#include "c_task.h"
#include "sleep.h"

#define SW_YEAR 0x2015
#define SW_MONTH 0x12
#define SW_DAY 0x09
#define SW_TYPE SANDBOX_SLEEP
#define SW_VERSION 0x0000

SoftwareIdentification softwareIdentification = {SW_YEAR, SW_MONTH, SW_DAY, SW_VERSION, SW_TYPE};
unsigned char VERSION_ID[] = "sandbox_sleep";

// Configuration Bit settings
// SYSCLK = "80 MHz (8MHz Crystal/ FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK = 40 MHz
// Primary Osc w/PLL (XT+,HS+,EC+PLL)
// WDT OFF
// Other options are don't care
//
    #pragma config UPLLEN   = ON        // USB PLL Enabled
    #pragma config FPLLMUL  = MUL_20        // PLL Multiplier
    #pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_2         // Peripheral Clock divisor
    #pragma config FWDTEN   = OFF           // Watchdog Timer
    //#pragma config WDTPS    = PS1           // Watchdog Timer Postscale Timeout is 1ms
    #pragma config WDTPS    = PS128         // Watchdog Timer Postscale Timeout is 128ms
    //#pragma config WDTPS    = PS1024        // Watchdog Timer Postscale Timeout is 1.024s
    #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect

unsigned char do_loopCnt = 0;
unsigned long loopCntTemp = 0;
unsigned long loopCnt = 0;
unsigned int tick_count = 0;

unsigned long loopCntHistory[60];
unsigned int loopCntHistoryCnt = 0;

int main (void) {
	DDPCON = 0; //Turn off JTAG and TRACE
	SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

	init();
	do_loopCnt = 0;

	while (1) {
		if (do_loopCnt) {
			do_loopCnt = 0;
			//lock_isr();
			loopCnt = loopCntTemp; //Risk is here that interrupt corrupts the value, but it is taken
			//unlock_isr();
			loopCntHistory[loopCntHistoryCnt] = loopCnt;
			loopCntHistoryCnt++;
			loopCntHistoryCnt %= ((sizeof(loopCntHistory)) / (sizeof(*loopCntHistory)));
			loopCntTemp = 0;
			if (loopCnt < 1000) { //1run/ms
				/*while (1) {
					Nop();
					ERROR_LED = 1;
				}*/
			}
		}

		do_task();
		do_app();
		do_io();
		do_tmr();

		loopCntTemp++;
	}
	return 0;
}

void isr_main_1ms(void) {
	static uint16 loopCntTmr = 0;

	tick_count++;

	loopCntTmr ++;
	if (loopCntTmr == 1000) {
		loopCntTmr = 0;
		do_loopCnt = 1;
	}
}
